/*
 * Copyright (C) 2015 Guido Berhoerster <guido+pwm@berhoerster.name>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "asprintf.h"

int
vasprintf(char **strp, const char *fmt, va_list args)
{
	int	len;
	va_list	args_new;
	char	*str;

	*strp = NULL;
	va_copy(args_new, args);

	len = vsnprintf(NULL, 0, fmt, args);
	if (len < 0) {
		goto out;
	}

	str = malloc(len + 1);
	if (str == NULL) {
		goto out;
	}
	len = vsnprintf(str, len + 1, fmt, args_new);
	if (len < 0) {
		free(str);
		goto out;
	}
	*strp = str;
out:
	va_end(args_new);

	return (len);
}

int
asprintf(char **strp, const char *fmt, ...)
{
	int	len;
	va_list	args;

	va_start(args, fmt);
	len = vasprintf(strp, fmt, args);
	va_end(args);

	return (len);
}
